<?php
/*
Plugin Name: Feeds Widget
Plugin URI: http://ketsugi.com/software/wordpress/feeds-widget-for-wordpress/
Description: A widget to display links to various context-sensitive RSS feeds.
Version: 1.1
Author: Joel Pan
Author URI: http://ketsugi.com/

  Copyright 2006  Joel Pan <spamtastic@ketsugi.com>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// This gets called at the plugins_loaded action
function widget_feeds_init () {

	// Check for the require API functions
	if ( !function_exists('register_sidebar_widget') || !function_exists('register_widget_control') )
		return;
	
	// This saves options and prints the widget's config form
	function widget_feeds_control () {
		$options = $newoptions = get_option('widget_feeds');
		if ( $_POST['feeds-submit'] ) {
			$newoptions['title'] = strip_tags(stripslashes($_POST['feeds-title']));
			$newoptions['protocol'] = strip_tags(stripslashes($_POST['feeds-protocol']));
		}
		if ( $options != $newoptions ) {
			$options = $newoptions;
			update_option('widget_feeds', $options);
		}
?>
				<div style="text-align: right">
					<label for="feeds-title" style="line-height:35px;display:block;"><?php _e('Widget title:', 'widgets'); ?> <input type="text" id="feeds-title" name="feeds-title" value="<?php echo wp_specialchars($options['title'], true); ?>" /></label>
					Protocol to use: <label><input type="radio" name="feeds-protocol" id="feeds-protocol-http" value="http"<?php if ("http" == $options['protocol']) echo ' checked="checked"' ?> /> http://</label> <label><input type="radio" name="feeds-protocol" id="feeds-protocol-feed" value="feed"<?php if ("feed" == $options['protocol']) echo ' checked="checked"' ?> /> feed://</label>
					<input type="hidden" name="feeds-submit" id="feeds-submit" value="1" />
				</div>				
<?php
	}
	// This prints the widget
	function widget_feeds ($args) {
		extract($args);
		$defaults = array('title'=>'&nbsp;', 'protocol'=>'feed');
		$options = (array) get_option('widget_feeds');

		foreach ( $defaults as $key => $value )
			if ( !isset($options[$key]) )
				$options[$key] = $defaults[$key];
		
		echo $before_widget;
		echo $before_title . $options['title'] . $after_title;
		
		$rss2_url = set_protocol(get_bloginfo('rss2_url'), $options['protocol']);
		$comments_rss2_url = set_protocol(get_bloginfo('comments_rss2_url'), $options['protocol']);
?>
	<ul>
		<li class="feed"><a href="<?php echo $rss2_url ?>">All Entries</a></li>
		<li class="feed"><a href="<?php echo $comments_rss2_url ?>">All Comments</a></li>
<?php
		//Context-sensitive RSS links
		if ( is_category() ) {
			$rss_link = get_rss_link( 'category', $options['protocol'] );
?>
		<li class="feed"><a href="<?php echo $rss_link ?>">This Category</a></li>
<?php
		}
		else if ( is_single() ) {
			$rss_link = get_rss_link( 'post', $options['protocol'] );		
?>
		<li class="feed"><a href="<?php echo $rss_link ?>">This Post's Comments</a></li>
<?php
		}
		else if ( is_search() ) {
			$rss_link = get_rss_link( 'search', $options['protocol'] );
?>
		<li class="feed"><a href="<?php echo $rss_link ?>">This Search</a></li>
<?php
		}
		else if ( function_exists( 'is_tag' ) && is_tag() ) {
			$rss_link = get_rss_link( 'tag', $options['protocol'] );
?>
		<li class="feed"><a href="<?php echo $rss_link ?>">This Tag</a></li>
<?php
		}
?>
	</ul>
<?php
		echo $after_widget;
	}
	
	// Tell Widgets about our new widget and its control
	register_sidebar_widget('Feeds', 'widget_feeds');
	register_widget_control('Feeds', 'widget_feeds_control');
}

/* Apart from the category link, getting the RSS links for posts and searches involves a lot of hacking at the request URI.
 * This should be considered TEMPORARY until proper function hooks are provided to do this.
 */
function get_rss_link ( $type, $protocol ) {
	$current_uri = parse_url( $_SERVER['REQUEST_URI'] );
	$current_uri['scheme'] = $protocol;
	$current_uri['host'] = $_SERVER['HTTP_HOST'];
	switch ( $type ) {
		case "category":
      //Get the category RSS
      global $wp_query;
      $cat_ID = $wp_query->get_queried_object_id();
      $rss_link = set_protocol(get_category_rss_link(0, $cat_ID, ''), $protocol);
			break;
		case "archive":
			break;
		case "post":
			if ( '' != get_settings( 'permalink_structure' ) ) {
			  $rss_link = trailingslashit( glue_url( $current_uri ) ) . 'feed/';
			}
			else {
			  $current_uri .= isset( $current_uri['query'] ) ? '&' : '';
			  $current_uri .= 'feed=rss2';
				$rss_link .= glue_url( $current_uri );
			}
			break;
		case "search":
			if ( '' != get_settings( 'permalink_structure' ) ) {
				parse_str( $current_uri['query'] );
				$rss_link = parse_url( trailingslashit( get_bloginfo( 'home' ) ) . 'search/' . urlencode($s) . '/feed/' );
				$rss_link['schema'] = $protocol;
				$rss_link = glue_url( $rss_link );
			}
			else {
				$current_uri['query'] .= '&feed=rss2';
				$rss_link = glue_url( $current_uri );
			}
			break;
		case "tag":
			if ( 'yes' == get_option( 'utw_use_pretty_urls' ) ) {
				$rss_link = trailingslashit( glue_url( $current_uri ) ) . 'feed/';
			}
			else {
				$current_uri['query'] .= '&feed=rss2';
				$rss_link = glue_url( $current_uri );
			}
	}
	return $rss_link;
}

function set_protocol ( $url, $protocol ) {
	$url = parse_url($url);
	$url['scheme'] = $protocol;
	return glue_url($url);
}


function glue_url ( $parsed ) {
	if (! is_array($parsed)) return false;
	$uri = isset($parsed['scheme']) ? $parsed['scheme'].':'.((strtolower($parsed['scheme']) == 'mailto') ? '':'//'): '';
	$uri .= isset($parsed['user']) ? $parsed['user'].($parsed['pass']? ':'.$parsed['pass']:'').'@':'';
	$uri .= isset($parsed['host']) ? $parsed['host'] : '';
	$uri .= isset($parsed['port']) ? ':'.$parsed['port'] : '';
	$uri .= isset($parsed['path']) ? $parsed['path'] : '';
	$uri .= isset($parsed['query']) ? '?'.$parsed['query'] : '';
	$uri .= isset($parsed['fragment']) ? '#'.$parsed['fragment'] : '';
	return $uri;
}

// Delay plugin execution to ensure Widgets has a chance to load first
add_action('widgets_init', 'widget_feeds_init');
